#Licensed Materials - Property of IBM 
#Rational ClearQuest 
#Copyright IBM Corp. 2008 All Rights Reserved 

package cqcli::controller;

use strict;
#use warnings;
require Exporter;

use Error qw(:try);
use cqcli::Exception;
use Inline::Java qw(caught cast) ;
use cqcli::Util;
use cqcli::commonservice;
use Text::ParseWords;
use Config;


use cqcli::editrecord;
use cqcli::controlcmd;
use cqcli::submitrecord;
use cqcli::addattachment;
use cqcli::removeattachment;
use cqcli::IOHandler;
use cqcli::findrecord;
use cqcli::newquery;
use cqcli::executequery;
use cqcli::deletequery;
use cqcli::renamequery;
use cqcli::movequery;
use cqcli::exportquery;
use cqcli::folder;
use cqcli::cqtm;
use cqcli::importquery;


##################################################################3
our @ISA = qw(Exporter);
our %EXPORT_TAGS = ( 'all' => [ qw() ] );
our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
our @EXPORT = qw(InitCQCLI GetCQCLIController);
our $VERSION = '0.01';
##########################################################
our $this = undef;
my $runscript = 0;

sub new 
{
	my $class  = shift;
	my $obj = {
		'ARGVBACKUP'			=> undef,
		'SIGWARNBACKUP'			=> undef,
		'CQCLICmdTable'			=> undef,
		'CQCLICmdTable_Alias'	=> undef,
		'CmdName'				=> undef,
	};
	
	bless $obj , ref($class)||$class;
	return $obj;
}

sub InitController()
{
	my $self = shift;
	InitCqcliCommonService();
	$self->InitCmdTable();
	InitAlias();
	return 1;
}


sub ConfigureGlobalSetting
{
	my $self  = shift;
	@{$self->{'ARGVBACKUP'}} = @ARGV;
	#$self->{'SIGWARNBACKUP'}=$SIG{__WARN__};
}

sub RestoreGlobalSetting()
{
	my $self = shift;
	if($self->{'ARGVBACKUP'}) {
		@ARGV = @{$self->{'ARGVBACKUP'}};
	}
	$self->{'CmdName'} = undef;

	return 1;
}

sub InitControllerForThisCmd()
{
	my $self = shift;
	my $cmd = shift;
	$self->{'CmdName'} = $cmd;
	$self->ConfigureGlobalSetting();
	return 1;
}

sub CleanUpControllerForThisCmd()
{
	my $self = shift;
	$self->RestoreGlobalSetting();
	if(IsCommandScopeSession()) {
		DestroySession();
		SetSessionScope(undef);
	}
	SetRequestObject(undef);
	SetResponseObject(undef);
	
}




sub InitCmdTable()
{ 
	my $self = shift;
	
	%{$self->{'CQCLICmdTable'}} = (
		'initparse'	  => 1,
		'edit_record' => \&editrecord,
		'submit'	  => \&submitrecord,
		'login'		  => \&login,
		'logout'	  => \&logout,
		'alias'		  => \&alias,
		'run_script'  => \&run_script,
		'status'	  => \&status,
		'cqclisetlog' => \&cqclisetlog,
		'add_attachment' => \&addattachment,
		'remove_attachment' => \&removeattachment,
		'find_record' => \&findrecord,
		'new_query' => \&newquery,
		'run_query' => \&runquery,
		'delete_query' => \&deletequery,
		'rename_query' => \&renamequery,
		'move_query'   => \&movequery,
		'export_query' => \&exportquery,
		'import_query' => \&importquery,
		'new_folder'   => \&newfolder,
		'rename_folder'   => \&renamefolder,
		'delete_folder'   => \&deletefolder,
		'move_folder'   => \&movefolder,
		'tmexecute'     =>\&tmexecute,
		'tmcommit'      =>\&tmcommit
		);

		%{$self->{'CQCLICmdTable_Alias'}} = (
			'er'	=>	'edit_record',
			'quit'  =>  'logout',
			'exit'  => 'logout',
			'sm'	=> 'submit',
			'submit_record'	=> 'submit',
			'attach' => 'add_attachment',
			'rem' => 'remove_attachment',
			'new_query' => 'new_query',
			'query' => 'run_query'
			);
		
		
		return 1;

}



sub UnknownCqcliCmd
{
	my ($self,@tmpCmd) = @_;

	#in future this can be extended to customization and extensibility framework
	ThrowException('CQCLI_Invalid_Cmd',$tmpCmd[0]);

}

sub check_line {
	my @line = @_;
	my $input = shift @line;
	$input = cqcliltrim($input);

	if($input =~ m/^#/ || !($input)){
		return 1;
	}
	else {
		return 0;
	}
}

sub alias
{
	my @cmdar = @_;
	my $cmd = shift @cmdar;
	my $aliasexpr = shift @cmdar;
	my ($aliasname , $cmdname) = (undef,undef);
	my ($cmdFound,$key) = (undef,undef);
	my $commandfile = undef;
	
	
	
	if(!($aliasexpr)){
		ThrowSyntaxErrorException('CQCLI_Missing_Argument',$cmd);
	}
	
	if($#cmdar >=0)
	{
		ThrowSyntaxErrorException('CQCLI_Err_UnrecognizeOpt',@cmdar);
	} 
	else {
		
		if($aliasexpr =~ /^[a-zA-Z][\w]*=[\w]+\b/ ) {
			($aliasname , $cmdname) = split(/=/,$aliasexpr);
			foreach $key (keys %{$this->{'CQCLICmdTable'}}) {
				if($key eq $cmdname) {
					$cmdFound = $key;
					last;
				}
			}
			if($cmdFound) {
				$this->{'CQCLICmdTable_Alias'}{$aliasname} = $cmdFound;
			}else {
				ThrowException('CQCLI_Invalid_Cmd',$cmdname);
			}

		}
		elsif(-e $aliasexpr){
					open(RIN, "<$aliasexpr")|| ThrowException('CQCLI_Failure_FileOpen',$aliasexpr);
						while (<RIN>) {
							$commandfile = $_;
							chomp($commandfile);
							
							if(check_line($commandfile)) {
							next;
							}
							
							($aliasname , $cmdname) = split(/=/,$commandfile);
							$cmdname = lc($cmdname);
							foreach $key (keys %{$this->{'CQCLICmdTable'}}) {
							if($key eq $cmdname) {
							$cmdFound = $key;
							last;
							}
						}
						if($cmdFound) {
						$this->{'CQCLICmdTable_Alias'}{$aliasname} = $cmdFound;
						$cmdFound = undef;
						}else {
						ThrowException('CQCLI_Invalid_Cmd',$cmdname);
						}
					}
					close(RIN)|| CLIPrintIdOutput('CQCLI_Info_FileClose',$aliasexpr);
		}
		else {
			ThrowSyntaxErrorException('CQCLI_Err_IllegalAliasExpr',$aliasexpr);
		}
	}
	return 1;

}

sub InitAlias {

	my @cmdar = @_;
	my $cmd = shift @cmdar;
	my $aliasexpr = shift @cmdar;
	my ($aliasname , $cmdname) = (undef,undef);
	my ($cmdFound,$key) = (undef,undef);
	my $commandfile = undef;

		my $aliasPropertiesFileName = undef;
		$aliasPropertiesFileName = $ENV{'CQCLI_CONFIG_DIR'};
		
		if($Config{osname} eq "MSWin32"){
			$aliasPropertiesFileName=$aliasPropertiesFileName."\\"."alias.init";
		}else{	#for unix platform	
			$aliasPropertiesFileName=$aliasPropertiesFileName."/"."alias.init";
		}

		open(RIN, "<$aliasPropertiesFileName");
			while (<RIN>) {
					$commandfile = $_;
					chomp($commandfile);
							
					if(check_line($commandfile)) {
						next;
					}
						
					($aliasname , $cmdname) = split(/=/,$commandfile);
					foreach $key (keys %{$this->{'CQCLICmdTable'}}) {
						if($key eq $cmdname) {
						$cmdFound = $key;
						last;
						}
					}
					if($cmdFound) {
					$this->{'CQCLICmdTable_Alias'}{$aliasname} = $cmdFound;
					$cmdFound = undef;
					}else {
						ThrowException('CQCLI_Invalid_Cmd',$cmdname);
					}
		}
		close(RIN)|| CLIPrintIdOutput('CQCLI_Info_FileClose',$aliasPropertiesFileName);
}


sub run_script {
	my ($inputcommand,$input) = @_;
	SetCQCLIInteractiveMode(0);
	my $commandfile = undef;
	my $output = undef;
	my $Controller = GetCQCLIController();
	my $session = GetSession();

	if($input eq  ""){
             ThrowException('CQCLI_Missing_FValue',"<filename>","run_script");
    }

	
		if (-e $input){
				open(RIN, "<$input")|| ThrowException('CQCLI_Failure_FileOpen',$input);
				while (<RIN>) {
					$commandfile = $_;
					chomp($commandfile);
					#Escape comment lines within the script file
					if(check_line($commandfile)) {
						next;
					}

					my @commandarray = split(' ',$commandfile);

					##########################################################################################################

					my $count;
					my $cliargv = "";
					my $sq = 0;
					my $token;

					for($count=0;$count<scalar(@commandarray);$count++){
			
					if($commandarray[$count] =~ /^(')(.+)/i || $commandarray[$count] =~ /^(")(.+)/i){
						$sq = 1;
						$token = $2 . " ";

			     		if(($commandarray[$count] =~ /'$/i) || ($commandarray[$count] =~ /"$/i)){
						$sq = 0;
						$token = $commandarray[$count];
						$token = substr($token,1,-1);
						goto FINISH;
					  }
					
					$count++;
					
					while($count<scalar(@commandarray)){
						if(($commandarray[$count] =~ /'$/i) || ($commandarray[$count] =~ /"$/i)) {
							$token = $token . $commandarray[$count];
							$sq = 0;
							$token = substr($token,0,-1);
							goto FINISH;
						}
						else {
							$token = $token . $commandarray[$count] . " ";
						}
						$count++;
					}

					if($sq eq 1){
						print "Can't find string terminator \"'\" anywhere before the end of command";exit 0;
					}
				
					}
					else {
						$token = $commandarray[$count];goto FINISH;
					}

	
					FINISH:$cliargv = $cliargv . $token . "#";
					}

					$cliargv = substr($cliargv,0,-1);

					##########################################################################################################

					$commandfile = $cliargv;

					$runscript = 1;
					$output = $Controller->execute($commandfile);
					$runscript = 0;
					

				}
			close(RIN)|| CLIPrintIdOutput('CQCLI_Info_FileClose',$input);					
		}
		else {
			ThrowException('CQCLI_Failure_FileOpen',$input);
		}
	
		SetCQCLIInteractiveMode(1);
}


sub execute
{
	my $self = shift;
	my $commandstr = shift;
	my $CommandOutput = undef;
	$CommandOutput = $self->ExecuteCommand($commandstr);
	return $CommandOutput;
}

sub ParseAndValidateCommand()
{
	my ($self,$commandstr) = @_;

	if(!($self->{'CQCLICmdTable'}{'initparse'})){
		$self->InitCmdTable();
	}
	my $cmdStr = $commandstr;
	my @cmdArray = ();



	if($cmdStr) {

		if(!IsInteractiveMode()){
			if(( $Config{osname} eq "MSWin32" ) || $runscript == 1){
				@cmdArray= parse_line('#',0,$cmdStr);
			}
			else {
				@cmdArray= parse_line('\s+',0,$cmdStr);
			}
		}
		else {
			@cmdArray= parse_line('\s+',0,$cmdStr);
		}

		$cmdArray[0] = lc $cmdArray[0];
	}else{
		ThrowException('CQCLI_Invalid_CmdStr');
	}
	if($self->{'CQCLICmdTable'}{$cmdArray[0]}) {
		$self->InitControllerForThisCmd($cmdArray[0]);
		$self->{'CQCLICmdTable'}{$cmdArray[0]}(@cmdArray);
	}elsif($self->{'CQCLICmdTable_Alias'}{$cmdArray[0]}) {
		$cmdArray[0] = $self->{'CQCLICmdTable_Alias'}{$cmdArray[0]};
		$self->InitControllerForThisCmd($cmdArray[0]);
		$self->{'CQCLICmdTable'}{$cmdArray[0]}(@cmdArray);
		
	}else {
		$self->{'CmdName'} = $cmdArray[0];
		$self->UnknownCqcliCmd(@cmdArray);
	}
	
	return 1;

}

sub ExecuteCommand
{
	my ($self , $cmdStr ) = @_;
	my $output = undef;
	my ($clisession , $requestObject , $responseObject) = (undef,undef,undef);
	try {
		$output = $self->ParseAndValidateCommand($cmdStr);
		if($output) {
			#do anything
			$output = undef;
		}
		if(GetRequestObject())
		{
			$clisession = GetSession();
			$requestObject = GetRequestObject();
			$responseObject = GetResponseObject();
			$responseObject = $clisession->executeCLICommand($requestObject);
			if($responseObject) {
				$output = $responseObject->getMessage("null");
			}
		}
		if($output) {

			if(IsAPIMode() || !IsInteractiveMode()) {
				return $output;
			}else {
				if(!($output)) {
					CLIPrintOutput($output."\n");
				}
			}
		}
		return 1;
	}
	catch cqcli::UsageException with {
		my $ex = shift;	
		$ex->printMessage();
		return 1;
	}
	catch cqcli::Exception with {
		my $ex = shift;	
		$ex->printMessage();
		if(!(IsAPIMode())){
			return 1;
		}
		return undef;
	}
	otherwise {
		my $tmpexp = undef;
		my $jmsg = undef;
		if (caught("java.lang.Exception"))
		{
			  $tmpexp = $@;	  
			  $jmsg = $tmpexp->getMessage();
			  my $msg = GetLocalizedMessage("CQCLI_Java_Error",$jmsg);
			  CLIPrintOutput($msg);
			  return 1;
		 }
	}
	finally { 
		$self->CleanUpControllerForThisCmd();
	};

}

##########################################################################################

sub InitCQCLI()
{
	if($this){
		return 1;
	}else {
		$this = new cqcli::controller();
		$this->InitController();
		return 1;
	}
	
}

sub GetCQCLIController()
{
	if(!$this) {
		InitCQCLI();
	}
	return $this;
}
############################################################################################

return 1;
__END__